import os
import numpy as np
import tensorflow as tf
from scipy.ndimage import gaussian_filter
from tensorflow.keras.layers.experimental.preprocessing import Rescaling
import math
import sys
from inputparameters import *
import matplotlib
import matplotlib.pyplot as plt
matplotlib.use("Agg")

def read_image(infile):

# This function reads fits couples of fits images.

    with open(infile, 'rb') as f1:
       image = np.fromfile(f1, dtype=np.float64)

       ## normalize data
       #image = Rescaling(np.amax(image), 0.0, "rescaling")(image)
       #image = np.clip(image/(0.9*np.amax(image)),0.0,1.0)
       image = np.clip(image,0.0,None)
       print(np.amin(image),np.amax(image))

    return [np.array(image,dtype=np.float64)]

def load_data_Images(seed,comm,image_set):

    extension = '.bin'
    print("Reading LABELS")

    numberofcontacts = endcontact
    numberofcontacts_eff = endcontact-startcontact+1
    numberofimages = numberofcontacts_eff*numberoffiles
    images = np.zeros((numberofimages,resolution_y,resolution_x))

# Build the training set

    print("Reading ",resolution_x,"x",resolution_y," TRAINING images")
    print("Total number of images = ",numberofimages)



    index = 0
    for icontacts in range(startcontact,endcontact+1):
      for i in range(image_set+1,image_set+numberoffiles+1):

        if verbose:
           print("Reading image N. ",i)
        #infile = imagepath+'/'+fileroot_image+str(i)+'_'+str(icontacts)+extension
        infile = imagepath+'/'+fileroot_image+str(i)+extension
        print(infile)
        # Load data
        image = read_image(infile)

        # calibrate
        if refvalues[index] > f_knee:
           images[index,:,:] = calibration_factor_high*np.reshape(image,(resolution_y,resolution_x))
           background = background_high
           if refvalues[0] == 1.0329:
               background = background_high + 0.07
        else:
           images[index,:,:] = calibration_factor_low*np.reshape(image,(resolution_y,resolution_x))
           background = background_low
           if refvalues[0] == 1.0329:
               background = 0.0

        # complete calibration and normalize [0:1]
        if background !=  100000.0:
            imax = 0.0
            imin = 1000.0
            for ii in range(resolution_y):
                for jj in range(resolution_x):
                    images[index,ii,jj] = images[index,ii,jj]+background
                    imax = max(imax,images[index,ii,jj])
                    imin = min(imin,images[index,ii,jj])
                    if images[index,ii,jj] < 0.0:
                        images[index,ii,jj] = 0.0
                    if images[index,ii,jj] > 1.0:
                        images[index,ii,jj] = 1.0
            if 0:#imax > 1.0 or imin < 0.0:
             print("STRETCHING: ",imax,imin)
             for ii in range(resolution_y):
                for jj in range(resolution_x):
                    images[index,ii,jj] = (images[index,ii,jj]-imin)/(imax-imin)
            images[index,:,:] = np.clip(images[index,:,:], 0, 1)
            print(images[index,:,:].max())

        # remove edge pixels
        if remove_edge_radius > 0:
           for ii in range(resolution_y):
                for jj in range(resolution_x):
                    dist = math.sqrt((ii-0.5*resolution_y)**2 + (jj-0.5*resolution_x)**2)
                    if dist > remove_edge_radius:
                       images[index,ii,jj] = 0.0

        # Add gaussian noise
        if sigmanoise > 0:
            dirtyimg = np.empty_like(images[index,:,:])
            dirtyimg = np.random.normal(loc=0.0, scale=sigmanoise, size=dirtyimg.shape)
            images[index,:,:] = np.clip(dirtyimg[:,:] + images[index,:,:], 0, 1)
        # Add gaussian blur
        if sigmablur > 0:
            dirtyimg = np.empty_like(images[index,:,:])
            gaussian_filter(input=images[index,:,:],sigma=sigmablur,output=dirtyimg,mode='constant',cval=0.0,truncate=4.0)
            images[index,:,:] = dirtyimg#+background

        # show image

        # write bin image
        if os.path.isdir('bin'):
           infile = "bin/calibrated_exp_"+str(i)+".bin"
           f1 = open(infile, 'wb')
           f1.write(images[index,:,:])
           f1.close()

        # write png
        if writeimages == 1:
            dpi = 1
            height = images[index,:,:].shape[0]
            width = images[index,:,:].shape[1]
            fig = plt.figure(figsize=(width, height),dpi=dpi)
            ax = fig.add_axes([0, 0, 1, 1])
            ax.axis('off')
            img1 = images[index,:,:]
            ax.imshow(np.transpose(img1), cmap='gray', interpolation='none')
            testfilename = "images/noise"+str(index)+".png"
            fig.savefig(testfilename,dpi=dpi)

        index = index+1

    return [images, numberofimages]
