# import the necessary packages
from tensorflow.keras.layers import BatchNormalization
from tensorflow.keras.layers import MaxPooling2D
from tensorflow.keras.layers import Dropout
from tensorflow.keras.layers import Conv2D
from tensorflow.keras.layers import Conv2DTranspose
from tensorflow.keras.layers import LeakyReLU
from tensorflow.keras.layers import Activation
from tensorflow.keras.layers import Flatten
from tensorflow.keras.layers import Dense
from tensorflow.keras.layers import Reshape
from tensorflow.keras.layers import Input
from tensorflow.keras.models import Model
from tensorflow.keras.layers import UpSampling2D
from tensorflow.keras.layers import add
from tensorflow.keras import backend as K
from tensorflow.keras.optimizers import Adam
import numpy as np
from inputparameters import *

class CNN:
        @staticmethod
        def build(width, height, depth, filters=(32, 64), eta=0.7, numberofcontacts=2):

                padd = "same"
                #padd = "valid"
         
		# initialize the input shape to be "channels last" along with
		# the channels dimension itself
		# channels dimension itself
                inputShape = (height, width, depth)
                chanDim = -1

		# define the input to the CNN 
                inputs = Input(shape=inputShape)
                x = inputs

		# loop over the number of filters
                index = 0
                for f in filters:
                        if index == 0:
                          # conv2D may be changed to SeparableConv2D
                          x = Conv2D(f, (3, 3), activation="relu", strides=1, padding=padd)(x)
                          x = BatchNormalization(axis=chanDim)(x)
                          previous_block_activation = x
                        else:
                          # apply a CONV => RELU => POOLING => BN operation
                          # first convolutional layer
                          x = Conv2D(f, (3, 3), activation="relu", strides=1, padding=padd)(x)
                          x = BatchNormalization(axis=chanDim)(x)
                          ### second convolutional layer
                          x = Conv2D(f, (3, 3), activation="relu", strides=1, padding=padd)(x)
                          x = BatchNormalization(axis=chanDim)(x)
                          ###
                          ### third convolutional layer
                          #x = Conv2D(f, (3, 3), activation="relu", strides=1, padding=padd)(x)
                          #x = BatchNormalization(axis=chanDim)(x)
                          ###
                          # Pooling
                          x = MaxPooling2D((2, 2), padding=padd)(x)
                          x = Dropout(eta)(x)
                          # Project residuals
                          residual = Conv2D(f, 1, strides=2, padding=padd)(previous_block_activation)
                          x = add([x, residual])
                          previous_block_activation = x

                        index = index+1

                # flatten the data array
                x = Flatten()(x)

                # fully connected layers
                units = 64
                activation = "relu"
                x = Dropout(eta)(x)
                x = Dense(units, activation=activation)(x)
                units = 32
                activation = "relu"
                x = Dropout(eta)(x)
                x = Dense(units, activation=activation)(x)
                units = 16
                activation = "relu"
                x = Dropout(eta)(x)
                x = Dense(units, activation=activation)(x)

                # output layer 
                Noutputs = numberofcontacts
                activation = "linear"#"softmax"
                x = Dropout(eta)(x)
                outputs = Dense(Noutputs, activation=activation)(x)

                cnn = Model(inputs, outputs, name="cnn")
                opt = Adam(lr=eta)
                cnn.compile(optimizer=opt, loss="mean_squared_error",metrics=["mean_squared_error"])#metrics=["accuracy"])

		# return the cnn
                return (cnn)
