import numpy as np
import tensorflow as tf
from scipy.ndimage import gaussian_filter
from tensorflow.keras.layers.experimental.preprocessing import Rescaling
import math
import sys
import random
from inputparameters import *
import matplotlib
import matplotlib.pyplot as plt
matplotlib.use("Agg")

h5io = 1
if h5io == 1:
    import h5py 

    def read_image_h5(infile,dsetname):
       image = infile[dsetname]
       print(np.amin(image),np.amax(image))

       return [np.array(image,dtype=np.float64)]
       

def read_image(infile):

    with open(infile, 'rb') as f1:
       image = np.fromfile(f1, dtype=np.float64)

       ## normalize data
       #image = Rescaling(1/255, 0.0, "rescaling")(image)
       #image = np.clip(image/np.amax(image),0.0,None)
       print(np.amin(image),np.amax(image))

    return [np.array(image,dtype=np.float64)]

def load_data(seed,comm,image_set,startcontact,endcontact):

    extension = '.bin'
    print("Reading LABELS")

    numberofcontacts = endcontact
    numberofcontacts_eff = endcontact-startcontact+1
    numberofimages = numberofcontacts_eff*numberoffiles
    images = np.zeros((numberofimages,resolution_y,resolution_x))
    values = np.zeros((numberofimages,numberofcontacts))

# Read the files containing the value of the forces and angles for the training and test sets
    values = np.zeros((numberofcontacts_eff*numberoffiles,numberofcontacts))
    testvalues = np.zeros((numberofcontacts_eff*numberoftests,numberofcontacts))

    index1 = 0
    index2 = 0
    for icontacts in range(startcontact,endcontact+1):
      infile = valuepath+'/'+valueroot+str(icontacts)
      f = open(infile,'r')
      if image_set > 0:
       for i in range(image_set):
        line = f.readline()
       for i in range(image_set,image_set+numberoffiles):
        line = f.readline()
        valstring = line.split()
        for j in range(numberofcontacts):
            values[index1,j] = float(valstring[j])
        index1 = index1+1
      else:    
       for i in range(numberoffiles):
        line = f.readline()
        valstring = line.split()
        for j in range(numberofcontacts):
            values[index1,j] = float(valstring[j])
        index1 = index1+1

       for i in range(numberoffiles,numberoffiles+numberoftests):
        line = f.readline()
        valstring = line.split()
        for j in range(numberofcontacts):
            testvalues[index2,j] = float(valstring[j])
        index2 = index2+1
       f.close()

# Build the training set

    print("Reading ",resolution_x,"x",resolution_y," TRAINING images")
    print("Total number of images = ",numberofimages)

    index = 0
    if h5io == 1:
        infile = imagepath+'/'+fileroot_image+str(icontacts)+'.h5'
        infileh5 = h5py.File(infile,'r')
    for icontacts in range(startcontact,endcontact+1):
      for i in range(image_set+1,image_set+numberoffiles+1):

        if verbose:
           print("Reading image N. ",i)
        if h5io == 0:
           infile = imagepath+'/'+fileroot_image+str(i)+'_'+str(icontacts)+extension
           print(infile)
           # Load data
           image = read_image(infile)
        if h5io == 1:
           dsetname = fileroot_image+str(i)+'_'+str(icontacts)
           image = read_image_h5(infileh5,dsetname)


        images[index,:,:] = np.reshape(image,(resolution_y,resolution_x))
        refvalue = np.amax(values[index,:], axis=0) 

        # Add edge
        for ii in range(resolution_y):
                for jj in range(resolution_x):
                    dist = math.sqrt((ii-0.5*resolution_y)**2 + (jj-0.5*resolution_x)**2)
                    if dist <= edge_radius and dist >= edge_radius-1:
                       edge_int = random.uniform(edge_brightness-0.1,edge_brightness+0.1)
                       images[index,ii,jj] = edge_int

        # Add gaussian noise
        if sigmanoise > 0:
            dirtyimg = np.empty_like(images[index,:,:])
            dirtyimg = np.random.normal(loc=0.0, scale=sigmanoise, size=dirtyimg.shape)
            images[index,:,:] = np.clip(dirtyimg[:,:] + images[index,:,:], 0, 1)
            ###images[index,:,:] = images[index,:,:]+background

        # Add background
        if background > 0.0:
            background0  = (background/fth)*refvalue
            if refvalue > fth:
             background0  = background
            dirtyimg = np.random.normal(loc=background0 , scale=sigmabackground, size=dirtyimg.shape)
            images[index,:,:] = np.clip(dirtyimg[:,:] + images[index,:,:], 0, 1)

        # Add gaussian blur
        if sigmablur > 0:
            dirtyimg = np.empty_like(images[index,:,:])
            gaussian_filter(input=images[index,:,:],sigma=sigmablur,output=dirtyimg,mode='constant',cval=0.0,truncate=4.0)
            images[index,:,:] = dirtyimg

        # Remove everything is outside edge+1pixel
        if remove_external_values == 1:
            edge_radius_abs = remove_edge_radius
            for ii in range(resolution_y):
                for jj in range(resolution_x):
                    dist = math.sqrt((ii-0.5*resolution_y)**2 + (jj-0.5*resolution_x)**2)
                    if dist > edge_radius_abs:
                       images[index,ii,jj] = 0.0

        # show image
        if writeimages == 1:
            dpi = 1
            height = images[index,:,:].shape[0]
            width = images[index,:,:].shape[1]
            fig = plt.figure(figsize=(width, height),dpi=dpi)
            ax = fig.add_axes([0, 0, 1, 1])
            ax.axis('off')
            ax.imshow(images[index,:,:], cmap='gray', interpolation='none')
            testfilename = "images/noise"+str(index)+".png"
            fig.savefig(testfilename,dpi=dpi)


        index = index+1
    if h5io == 1:
        infileh5.close()

# Build the test set

    numberoftestimages = numberofcontacts_eff*numberoftests
    testimages = np.zeros((numberoftestimages,resolution_y,resolution_x))

    print("Reading ",resolution_x,"x",resolution_y," TEST images")
    print("Total number of images = ",numberoftestimages)

    index = 0
    if h5io == 1:
        infile = imagepath+'/'+fileroot_image+str(icontacts)+'.h5'
        infileh5 = h5py.File(infile,'r')
    for icontacts in range(startcontact,endcontact+1):
      for i in range(numberoffiles+1,numberoffiles+numberoftests+1):

        if verbose:
           print("Reading image N. ",i)
        if h5io == 0:
           infile = imagepath+'/'+fileroot_image+str(i)+'_'+str(icontacts)+extension
           print(infile)
           # Load data
           image = read_image(infile)
        if h5io == 1:
           dsetname = fileroot_image+str(i)+'_'+str(icontacts)
           image = read_image_h5(infileh5,dsetname)

        testimages[index,:,:] = np.reshape(image,(resolution_y,resolution_x))
        refvalue = np.amax(testvalues[index,:], axis=0)

        # Add edge
        for ii in range(resolution_y):
                for jj in range(resolution_x):
                    dist = math.sqrt((ii-0.5*resolution_y)**2 + (jj-0.5*resolution_x)**2)
                    if dist <= edge_radius and dist >= edge_radius-1:
                       edge_int = random.uniform(edge_brightness-0.1,edge_brightness+0.1)
                       testimages[index,ii,jj] = edge_int
        # Add gaussian noise
        if sigmanoise > 0:
            dirtyimg = np.empty_like(testimages[index,:,:])
            dirtyimg = np.random.normal(loc=0.0, scale=sigmanoise, size=dirtyimg.shape)
            testimages[index,:,:] = np.clip(dirtyimg[:,:] + testimages[index,:,:], 0, 1)

        # Add background
        if background > 0.0:
            background0  = (background/fth)*refvalue
            if refvalue > fth:
             background0  = background
            dirtyimg = np.random.normal(loc=background0 , scale=sigmabackground, size=dirtyimg.shape)
            testimages[index,:,:] = np.clip(dirtyimg[:,:] + testimages[index,:,:], 0, 1)

        # Add gaussian blur
        if sigmablur > 0:
            dirtyimg = np.empty_like(testimages[index,:,:])
            gaussian_filter(input=testimages[index,:,:],sigma=sigmablur,output=dirtyimg,mode='constant',cval=0.0,truncate=4.0)
            testimages[index,:,:] = dirtyimg+background

        # Remove everything is outside edge+1pixel
        if remove_external_values == 1:
            edge_radius_abs = edge_radius+1
            for ii in range(resolution_y):
                for jj in range(resolution_x):
                    dist = math.sqrt((ii-0.5*resolution_y)**2 + (jj-0.5*resolution_x)**2)
                    if dist > edge_radius_abs:
                      testimages[index,ii,jj] = 0.0

        # show image
        if writeimages == 1:
            dpi = 1
            height = testimages[index,:,:].shape[0]
            width = testimages[index,:,:].shape[1]
            fig = plt.figure(figsize=(width, height),dpi=dpi)
            ax = fig.add_axes([0, 0, 1, 1])
            ax.axis('off')
            ax.imshow(testimages[index,:,:], cmap='gray', interpolation='none')
            testfilename = "images/noise"+str(index)+".png"
            fig.savefig(testfilename,dpi=dpi)


        index = index+1

    if h5io == 1:
        infileh5.close()

    return [images, testimages, values, testvalues, numberofimages, numberoftestimages]
