# set the matplotlib backend so figures can be saved in the background
import matplotlib
matplotlib.use("Agg")

# import the necessary packages
from cnnmodel import CNN
from tensorflow import keras
from tensorflow.keras.optimizers import Adam
from tensorflow.keras.optimizers import RMSprop
from tensorflow.keras.wrappers.scikit_learn import KerasRegressor
import matplotlib.pyplot as plt
import numpy as np
import argparse
import glob
import math
import sys
import time
import fileinput
from inputparameters import *
from mpi_routines import *
from  load_data import *

tf.get_logger().setLevel('WARNING') #DEBUG, INFO, WARNING, ERROR
# construct the argument parse and parse the arguments
# parse command line arguments
time1 = time.time()
parser = argparse.ArgumentParser()
parser.add_argument('--hyper_file', type=str, help='Read the hyperparameter file')
parser.add_argument('--ngpus', type=int, help='Number of gpus per node', default=1)
parser.add_argument('--rand', type=int, help='Random seed')
parser.add_argument('--ncontacts', type=int, help='Number of contacts', default=-1)
#parser.add_argument('--model', type=str, help='cnn, dda', default="none")
args = parser.parse_args()
seed = args.rand

if args.ncontacts <= 1:
    startcontact = startcontact0
    endcontact = endcontact0
else:
    startcontact = args.ncontacts
    endcontact = args.ncontacts
numberofcontacts = endcontact

if USE_MPI == 1:
   comm = MPI.COMM_WORLD
   rank = comm.rank
   comm.bcast(seed,0)
   comm.bcast(numberofcontacts,0)
   comm.bcast(startcontact,0)
   comm.bcast(endcontact,0)
else:
   comm = 0
   rank = 0


print('===============================================')
print('        NUMBER OF CONTACTS = ',startcontact,endcontact)
print('===============================================')

# load json file and distribute parameters
hyper_json = read_json(args.hyper_file)
if USE_MPI == 1:
   hyper_dict = distribute_parameters(hyper_json, comm)
else:
   hyper_dict = distribute_parameters(hyper_json)

# set GPU environment
if args.ngpus > 0:
    config_proto = set_gpus()
    if USE_MPI == 1:
       mydevice = '/device:GPU:'+str(divmod(comm.rank, args.ngpus)[1])
    else:
       mydevice = '/device:GPU:'+str(divmod(0, args.ngpus)[1])
    config_proto.gpu_options.allow_growth = True
else:
    config_proto = tf.compat.v1.ConfigProto()
    if USE_MPI == 1:
       mydevice = '/device:CPU:'+str(comm.rank)
    else:
       mydevice = '/device:CPU:'+str(0)
    #config_proto.gpu_options.allow_growth = True


# Read Data
time2 = time.time()
tt1 = np.zeros(1,dtype=int)
tt2 = np.zeros(1,dtype=int)

# Create input arrays
#trainX = np.zeros((trainsize,tile_resolution_y,tile_resolution_x),dtype=np.float64)
#testX = np.zeros((testsize,tile_resolution_y,tile_resolution_x),dtype=np.float64)
#trainXNoisy = np.zeros((trainsize,tile_resolution_y,tile_resolution_x),dtype=np.float64)
#testXNoisy = np.zeros((testsize,tile_resolution_y,tile_resolution_x),dtype=np.float64)

if rank == 0:
   trainXNoisy, testXNoisy, trainX, testX, trainsize, testsize = load_data(seed,comm,0,startcontact,endcontact)
   tt1[0] = trainsize
   tt2[0] = testsize
else:
   trainsize = 0
   testsize = 0
if USE_MPI == 1:
   comm.Bcast(tt1,root=0)
   comm.Bcast(tt2,root=0)
   comm.barrier() #remove this at the end!
   trainsize = tt1[0]
   testsize = tt2[0]

if rank > 0:
   trainX = np.empty((trainsize,numberofcontacts),dtype=np.float64)
   testX = np.empty((testsize,numberofcontacts),dtype=np.float64)
   trainXNoisy = np.empty((trainsize,resolution_y,resolution_x),dtype=np.float64)
   testXNoisy = np.empty((testsize,resolution_y,resolution_x),dtype=np.float64)

print(rank,trainX.shape, testX.shape, trainXNoisy.shape, testXNoisy.shape)

#trainX = testX_aux[:trainsize,:]
#testX = testX_aux[trainsize:,:]

# Broadcast data (in case of MPI)
if USE_MPI == 1:
   comm.Bcast([trainX, MPI.DOUBLE], root=0)
   comm.barrier()
   comm.Bcast([testX, MPI.DOUBLE], root=0)
   comm.barrier()
   comm.Bcast([trainXNoisy, MPI.DOUBLE], root=0)
   comm.barrier()
   comm.Bcast([testXNoisy, MPI.DOUBLE], root=0)
   comm.barrier()

# resize arrays
#trainX = np.expand_dims(trainX, axis=-1)
#testX = np.expand_dims(testX, axis=-1)
trainXNoisy = np.expand_dims(trainXNoisy, axis=-1)
testXNoisy = np.expand_dims(testXNoisy, axis=-1)

if USE_MPI == 1:
   comm.barrier()

# construct CNN
print("[INFO] building CNN...")

time3 = time.time()
index = 0
hyperlist = []

#with tf.device(mydevice):
with tf.compat.v1.Session(config=config_proto) as sess:
     for imodel in hyper_dict['hyperparam']:
         #with tf.device(mydevice):
              num_epoch = int(imodel['num_epoch'])
              nbatch = int(imodel['nbatch'])
              eta_input = float(imodel['eta_input'])
              dense_layer = int(imodel['dense_layer'])
              deep_model = int(imodel['deepmodel'])
              trains_per_epoch = int(trainsize/nbatch)
              print(">>>>>>>>>>>>>>>> ",trains_per_epoch)
              checkpoint_path = 'model-'+str(startcontact)+'-'+valueroot+str(deep_model)+'-'+str(nbatch)+'-'+'{epoch:03d}'+'-'+str(eta_input)+'.ckpt'
              #########checkpoint_path = "cp-{epoch:04d}.ckpt"
              cp_callback = tf.keras.callbacks.ModelCheckpoint(filepath=checkpoint_path,verbose=1,save_freq=250*trains_per_epoch)

              print("MPI Rank ",rank," running model ",index," with ",num_epoch," epochs, ",nbatch, " as batch size, "\
                    ,eta_input, " as learning rate and ",deep_model," as CNN Model")

              if regressor_type == 'cnn':
                 cnn = CNN.build(resolution_y, resolution_x, 1, (32, 64, 128), eta_input, numberofcontacts)
                 cnn.fit(trainXNoisy, trainX, 
                         validation_data=(testXNoisy, testX),
                         epochs=num_epoch,
                         batch_size=nbatch,callbacks=[cp_callback])
                 y_pred = cnn.predict(testXNoisy)
              
              elif regressor_type == 'keras':
                 # for 32/63 px images
                 #regressor = KerasRegressor(build_fn= lambda: CNN.build(resolution_y, resolution_x, 1, (32, 64, 128, 256), eta_input),epochs=num_epoch,batch_size=nbatch)
                 # for 16 px images
                 regressor = KerasRegressor(build_fn= lambda: CNN.build(resolution_y, resolution_x, 1, (32, 64, 128), eta_input, numberofcontacts),epochs=num_epoch,batch_size=nbatch)
                 regressor.fit(trainXNoisy, trainX, validation_data=(testXNoisy, testX),callbacks=[cp_callback])
                 y_pred = regressor.predict(testXNoisy)

              error = 0.0
              print(y_pred.shape)
              for j in range(y_pred.shape[0]):
                  for i in range(y_pred.shape[1]):
                      error = error+abs(y_pred[j,i]-testX[j,i])
              #error = math.sqrt(error)
              #mse_krr = mean_squared_error(testX, y_pred)

              # Control values
              controlfile = 'control-'+str(startcontact)+'-'+str(rank)+'-'+valueroot+str(deep_model)+'-'+str(nbatch)+'-'+str(num_epoch)+'-'+str(eta_input)+'.txt'
              idy = np.arange((endcontact-startcontact+1)*numberoftests)
              rdmy = np.random.RandomState(seed)
              rdmy.shuffle(idy)

              f = open(controlfile,'w')
              f.write(str(error/y_pred.size)+'\n')
              for j in range(100):
                  for i in range(y_pred.shape[1]):
                      f.write(str(y_pred[idy[j],i])+' '+str(testX[idy[j],i])+'\n')
                  #f.write("=========================="+'\n')
              f.close()

              print("ERROR = ",error/y_pred.size)

              if checkpoint == 1:
                restartfile = 'model-'+str(startcontact)+'-'+valueroot+str(deep_model)+'-'+str(nbatch)+'-'+str(num_epoch)+'-'+str(eta_input)+'.ckpt'
                if regressor_type == 'cnn':
                 cnn.save(restartfile)
                elif regressor_type == 'keras':
                 regressor.model.save(restartfile)

time4 = time.time()

if rank == 0:
        print("Set-up time     = ",time2-time1)
        print("Data load time  = ",time3-time2)
        print("Processing time = ",time4-time3)
        print("Total time      = ",time4-time1)
